
#include "TestTable.h"

#include <QxMemLeak.h>

QX_REGISTER_CPP_TEST(TestTable);
QX_REGISTER_CPP_TEST(DateTime);


using namespace std;

// Possible QDateTime Format  Result
// dd.MM.yyyy                 21.05.2001
// ddd MMMM d yy              Tue May 21 01
// hh:mm:ss.zzz               14:13:09.042
// h:m:s ap                   2:13:9 pm
// 
// Cross-database format:
// yyyyMMddhhmmss
// 
// User-defined format:
// yyyy.MM.dd-hh:mm:ss
const char* DateTime::format_ = "yyyy.MM.dd-hh:mm:ss";

namespace qx {
namespace cvt {
namespace detail {

template <> struct QxStringCvt_ToVariant< DateTime > 
{
    static inline QVariant toVariant(const DateTime &t,
                                     const QString  &format,
                                     int             index)
    {
        Q_UNUSED(format); Q_UNUSED(index); 
        cout << "Inside ToVariant" << endl;
        cout << t.toString() << endl;
        cout << format.toStdString() << endl;
        QDateTime qdt = qdt.fromString(t.toString().c_str(), format);
        cout << qdt.toString().toStdString() << endl;
        cout << "End of ToVariant" << endl;
        return qdt;
    }
};

template <> struct QxStringCvt_FromVariant< DateTime > 
{
    static inline qx_bool fromVariant(const QVariant &v,
                                      DateTime       &t,
                                      const QString  &format,
                                      int             index)
    { 
        Q_UNUSED(format); Q_UNUSED(index); 
        QDateTime qdt(v.toDateTime());
        cout << "Inside FromVariant" << endl;
        cout << qdt.toString().toStdString() << endl;
        cout << format.toStdString() << endl;
        t.fromString(qdt.toString(format).toStdString());
        cout << t.toString() << endl;
        cout << "End of FromVariant" << endl;
        return qx_bool(true);
    }
};

} // namespace detail
} // namespace cvtv

template <> void register_class(QxClass<DateTime> & t)
{
    t.data(&DateTime::datetime_, "datetime");
}

template <> void register_class(QxClass<TestTable> & t)
{
    cout << "Setting table name..." << endl;
    t.setName("testTable");
    t.id(&TestTable::id_, "id");
    IxDataMember *p = t.data(&TestTable::updateTime_, "updateTime");
    cout << "Setting datetime format to: " << DateTime::format() << endl;
    p->setFormat(DateTime::format());
    t.fctStatic_1<int, int>(&TestTable::square, "square");
    t.fct_0<DateTime>(&TestTable::getUpdateTime, "getUpdateTime");
    t.fct_2<string, const string&, char*>(&TestTable::concat, "concat");
}

} // namespace qx

int TestTable::square(int i)
{
    return i * i;
}

DateTime TestTable::getUpdateTime() const
{
    return updateTime_;
}

string TestTable::concat(const string& sep, const char *str)
{
    return id_ + sep + str;
}

