#include "precompiled.h"

#include "TestTable.h"

using namespace std;

int main(int argc, char** argv)
{
    Q_UNUSED(argc);
    Q_UNUSED(argv);

    cout << "Started..." << endl;

    QSqlError daoError;

    //////////////// Init Database ////////////////
    qx::QxSqlDatabase::getSingleton()->setDriverName("QMYSQL");
    qx::QxSqlDatabase::getSingleton()->setDatabaseName("db_marat");
    qx::QxSqlDatabase::getSingleton()->setHostName("localhost");
    //qx::QxSqlDatabase::getSingleton()->setHostName("qfrlnxaf");
    qx::QxSqlDatabase::getSingleton()->setUserName("qfr");
    qx::QxSqlDatabase::getSingleton()->setPassword("qfr");

    cout << "DB connected..." << endl;

    //////////// Remove previous ABC record ///////////////////
    {
        TestTablePtr criteriaPtr(new TestTable); 
        criteriaPtr->id_ = "ABC";
        daoError = qx::dao::delete_by_id(criteriaPtr);
    }

    /////////////// Insert New ABC record /////////////////
    {
        TestTablePtr rowPtr(new TestTable);
        rowPtr->id_ = "ABC"; 
        QDateTime currQdt(QDateTime::currentDateTime());
        string qdtStr(currQdt.toString(DateTime::format()).toStdString());
        rowPtr->updateTime_.fromString(qdtStr);
        daoError = qx::dao::insert(rowPtr);
    }

    //////////// Remove previous DEF record ///////////////////
    {
        TestTablePtr criteriaPtr(new TestTable); 
        criteriaPtr->id_ = "DEF";
        daoError = qx::dao::delete_by_id(criteriaPtr);
    }

    /////////////// Insert New DEF record /////////////////
    {
        cout << "---------------------------" << endl;
        cout << "Testing creation by reflection" << endl;

        // Creation
        TestTablePtr tablePtr(qx::create_nude_ptr<TestTable>("TestTable"));
        
        // Preparing input
        QDateTime qdt(QDateTime::currentDateTime());
        cout << qdt.toString().toStdString() << endl;
        DateTime updt;
        updt.fromString(qdt.toString(DateTime::format()).toStdString());

        // const char* does not work!
        //const char *idStr = "DEF";
        std::string idStr("DEF");
        const boost::serialization::nvp<std::string> _id("id", idStr);
        const boost::serialization::nvp<DateTime> _updt("updateTime", updt);

        // Population
        qx::IxClass *type = qx::QxClassX::getClass("TestTable");
        qx::IxDataMemberX *memberLst = type->getDataMemberX();

        qx::IxDataMember *pId = memberLst->get(_id.name());
        assert(pId);
        bool res = pId->setValue(tablePtr.get(), _id.value());
        assert(res);
        cout << "tablePtr->id_ = " << tablePtr->id_ << endl;

        qx::IxDataMember *pUpdt = memberLst->get(_updt.name());
        assert(pUpdt);
        pUpdt->setValue(tablePtr.get(), _updt.value());
        assert(res);
        cout << "tablePtr->updateTime_ = " << 
            tablePtr->updateTime_.toString() << endl;

        // Function
        qx::IxFunctionX *fctList = type->getFctMemberX(); 
        string key("getUpdateTime");
        if (!tablePtr || !fctList || !fctList->exist(key.c_str())) {
            cout << key.c_str() << " non-existent" << endl;
            throw;
        }
        qx::IxFunction_ptr fct0 = fctList->getByKey(key.c_str());
        boost::any ret;
        bool success = qx::QxClassX::invoke("TestTable", key.c_str(),
                                            *tablePtr, QString(), &ret);
        DateTime upt = boost::any_cast<DateTime>(ret);
        cout << "getUpdateTime() v0 = " << upt.toString() << endl;

        success = qx::QxClass<TestTable>::invoke(key.c_str(),
                                                 tablePtr.get(), 
                                                 QString(), &ret);
        DateTime upt1 = boost::any_cast<DateTime>(ret);
        cout << "getUpdateTime() v1 = " << upt1.toString() << endl;

        success = fct0->invoke(tablePtr.get(), QString(), &ret);
        DateTime upt2 = boost::any_cast<DateTime>(ret);
        cout << "getUpdateTime() v2 = " << upt2.toString() << endl;

        success = fct0->invoke(tablePtr.get(), vector<boost::any>(), &ret);
        DateTime upt3 = boost::any_cast<DateTime>(ret);
        cout << "getUpdateTime() v3 = " << upt3.toString() << endl;

        // Static Function
        key = "square";
        qx::IxFunctionX *fctStaticList = type->getFctStaticX(); 
        if (!fctList->exist(key.c_str())) {
            cout << "static method " << key.c_str() 
                 << " not in member function list" << endl;
        }
        if (!fctStaticList->exist(key.c_str())) {
            cout << "static method " << key.c_str() 
                 << " not in static function list" << endl;
            throw;
        }
        qx::IxFunction_ptr fct1 = fctStaticList->getByKey(key.c_str());
        int intRet;
        int a = 3;
        vector<boost::any> params(1);
        params[0] = a;
        success = qx::QxClassX::invokeStatic("TestTable", key.c_str(),
                                             params, &ret);
        intRet = boost::any_cast<int>(ret);
        cout << "square(" << a << ") v0 = " << intRet << endl;

        success = qx::QxClass<TestTable>::invokeStatic(key.c_str(),
                                                       params,
                                                       &ret);
        intRet = boost::any_cast<int>(ret);
        cout << "square(" << a << ") v1 = " << intRet << endl;

        success = fct1->invoke(params, &ret);
        intRet = boost::any_cast<int>(ret);
        cout << "square(" << a << ") v2 = " << intRet << endl;

        // Function with 2 parameters
        key = "concat";        // Function with 2 parameters
        if (!fctList->exist(key.c_str())) {
            cout << key.c_str() << " non-existent" << endl;
            throw;
        }
        qx::IxFunction_ptr fct2 = fctList->getByKey(key.c_str());
        params.resize(2);
        string sep("-");
        char *str = "tail";
        //string str("tail");
        params[0] = sep;
        params[1] = str;
        success = fct2->invoke(tablePtr.get(), params, &ret);
        string strRet = boost::any_cast<string>(ret);
        cout << "concat(" << sep << ", " << str << ") = " << strRet << endl;

        // Verification of serialization and persistence
        daoError = qx::dao::insert(tablePtr);
        cout << "---------------------------" << endl;
    }

    //////////// Select  Query rows //////////////////////
    listTestTable rowsTestTable;
    qx::QxSqlQuery query("where id = 'ABC' or id = 'DEF'");
    cout << "performing query" << endl;
    daoError = qx::dao::fetch_by_query(query, rowsTestTable);
    //cout << "Returned " << rowsTestTable.count() << " rows " << endl;
    cout << "Returned " << rowsTestTable.size() << " rows " << endl;
    //qx::dump(rowsTestTable); // will create xml dump
    for (unsigned int i = 0; i < rowsTestTable.size(); ++i)
    {
        TestTablePtr rowPtr = rowsTestTable[i];
        //cout << "id=" << rowPtr->id_.toStdString() 
        //     << " updateTime=" << rowPtr->updateTime_.toString().toStdString() << endl;
        //cout << "id=" << rowPtr->id_
        //     << " updateTime=" << rowPtr->updateTime_.toString().toStdString() << endl;
        cout << "id=" << rowPtr->id_
             << " updateTime=" << rowPtr->updateTime_.toString() << endl;
    }

    return 0;
}
